@@ -16,15 +16,14 @@ class SQLDbReader:
             if not self.file_exists:
                 raise FileNotFoundError(str(self.path) + " is not a valid path. File does not exist.")
 
-    def perform_read_query(self, query: str, params: dict):
+    def perform_read_query(self, query: str, params=None):
+        if params is None:
+            params = {}
         self.file_exists = os.path.isfile(self.path)
         if not self.file_exists:
             raise FileNotFoundError(str(self.path) + " is not a valid path. File does not exist.")
 
-        con = sqlite3.connect(self.path)
-        cur = con.cursor()
-        data = []
-        for row in cur.execute(query, params):
-            data.append(row)
-        con.close()
+        with sqlite3.connect(self.path) as con:
+            cur = con.cursor()
+            data = [row for row in cur.execute(query, params)]
         return data
